# Created byMartin.cz
# Copyright (c) Martin Strohalm. All rights reserved.

# See the Rebrickable API documentation at https://rebrickable.com/api/v3/docs/

from . import config
from .request import request


def get_categories(page=None, page_size=None, ordering=None, api_key=None):
    """
    Gets details for all available part categories.
    
    Args:
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    path = config.API_LEGO_URL + "part_categories/"
    
    return request(path, parameters)


def get_category(category_id, api_key=None):
    """
    Gets details about specific part category.
    
    Args:
        category_id: str or int
            Rebrickable category id.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {'key': api_key}
    
    path = config.API_LEGO_URL + "part_categories/%s/" % category_id
    
    return request(path, parameters)


def get_colors(page=None, page_size=None, ordering=None, api_key=None):
    """
    Gets details for all available colors.
    
    Args:
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    path = config.API_LEGO_URL + "colors/"
    
    return request(path, parameters)


def get_color(color_id, api_key=None):
    """
    Gets details about specific color.
    
    Args:
        color_id: str or int
            Rebrickable color id.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {'key': api_key}
    
    path = config.API_LEGO_URL + "colors/%s/" % color_id
    
    return request(path, parameters)


def get_element(element_id, api_key=None):
    """
    Gets details about specific element.
    
    Args:
        element_id: str or int
            Rebrickable element id.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {'key': api_key}
    
    path = config.API_LEGO_URL + "elements/%s/" % element_id
    
    return request(path, parameters)


def get_moc(moc_id, api_key=None):
    """
    Gets details about specific MOC.
    
    Args:
        moc_id: str or int
            Rebrickable MOC id.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    if 'MOC' not in str(moc_id):
        moc_id = "MOC-%s" % moc_id
    
    parameters = {'key': api_key}
    
    path = config.API_LEGO_URL + "mocs/%s/" % moc_id
    
    return request(path, parameters)


def get_moc_elements(moc_id, part_details=False, page=None, page_size=None, ordering=None, api_key=None):
    """Gets list of elements for a specific MOC.
    
    Args:
        moc_id: str or int
            Rebrickable MOC id.
        
        part_details: bool
            If set to True part details will be retrieved.
        
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    if 'MOC' not in str(moc_id):
        moc_id = "MOC-%s" % moc_id
    
    parameters = {
        'inc_part_details': int(part_details) or None,
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    path = config.API_LEGO_URL + "mocs/%s/parts/" % moc_id
    
    return request(path, parameters)


def get_parts(search=None, part_id=None, part_ids=None, part_cat_id=None, color_id=None, bricklink_id=None, brickowl_id=None, lego_id=None, ldraw_id=None, part_details=False, page=None, page_size=None, ordering=None, api_key=None):
    """
    Gets details for all available parts with optional filters.
    
    Args:
        search: str, int or None
            Search term e.g. part ID or name.
        
        part_id: str, int or None
            Rebrickable part ID.
        
        part_ids: (str,), (int,) or None
            Rebrickable part IDs.
        
        part_cat_id: str, int or None
            Rebrickable category ID.
        
        color_id: str, int or None
            Rebrickable color ID.
        
        bricklink_id: str, int or None
            BrickLink part ID.
        
        brickowl_id: str, int or None
            BrickOwl part ID.
        
        lego_id: str, int or None
            LEGO part ID.
        
        ldraw_id: str, int or None
            LDraw part ID.
        
        part_details: bool
            If set to True part details will be retrieved.
        
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    if part_ids:
        part_ids = ",".join(str(i) for i in part_ids)
    
    parameters = {
        'search': search,
        'part_num': part_id,
        'part_nums': part_ids,
        'part_cat_id': part_cat_id,
        'color_id': color_id,
        'bricklink_id': bricklink_id,
        'brickowl_id': brickowl_id,
        'lego_id': lego_id,
        'ldraw_id': ldraw_id,
        'inc_part_details': int(part_details) or None,
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    path = config.API_LEGO_URL + "parts/"
    
    return request(path, parameters)


def get_part(part_id, api_key=None):
    """
    Gets details about specific part.
    
    Args:
        part_id: str or int
            Rebrickable part id.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {'key': api_key}
    
    path = config.API_LEGO_URL + "parts/%s/" % part_id
    
    return request(path, parameters)


def get_part_color(part_id, color_id, api_key=None):
    """
    Gets details about specific part/color combination.
    
    Args:
        part_id: str or int
            Rebrickable part id.
        
        color_id: str or int
            Rebrickable color id.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {'key': api_key}
    
    path = config.API_LEGO_URL + "parts/%s/colors/%s/" % (part_id, color_id)
    
    return request(path, parameters)


def get_part_colors(part_id, page=None, page_size=None, ordering=None, api_key=None):
    """
    Gets details about available colors for specific part.
    
    Args:
        part_id: str or int
            Rebrickable part id.
        
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    path = config.API_LEGO_URL + "parts/%s/colors/" % part_id
    
    return request(path, parameters)


def get_part_color_sets(part_id, color_id, page=None, page_size=None, ordering=None, api_key=None):
    """
    Gets details about available sets containing specific part/color
    combination.
    
    Args:
        part_id: str or int
            Rebrickable part id.
        
        color_id: str or int
            Rebrickable color id.
        
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    path = config.API_LEGO_URL + "parts/%s/colors/%s/sets/" % (part_id, color_id)
    
    return request(path, parameters)


def get_sets(search=None, theme_id=None, min_year=None, max_year=None, min_pieces=None, max_pieces=None, page=None, page_size=None, ordering=None, api_key=None):
    """
    Gets details for all available sets with optional filters.
    
    Args:
        search: str, int or None
            Search term e.g. set ID or name.
        
        theme_id: str, int or None
            Rebrickable theme ID.
        
        min_year: int or None
            Minimum allowed release year.
        
        max_year: int or None
            Maximum allowed release year.
        
        min_pieces: int or None
            Minimum number of pieces.
        
        max_pieces: int or None
            Maximum number of pieces.
        
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'search': search,
        'theme_id': theme_id,
        'min_year': min_year,
        'max_year': max_year,
        'min_parts': min_pieces,
        'max_parts': max_pieces,
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    path = config.API_LEGO_URL + "sets/"
    
    return request(path, parameters)


def get_set(set_id, api_key=None):
    """
    Gets details about specific set.
    
    Args:
        set_id: str or int
            Rebrickable set id.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    if '-' not in str(set_id):
        set_id = "%s-1" % set_id
    
    parameters = {'key': api_key}
    
    path = config.API_LEGO_URL + "sets/%s/" % set_id
    
    return request(path, parameters)


def get_set_elements(set_id, part_details=False, page=None, page_size=None, ordering=None, api_key=None):
    """
    Gets details about available elements for specific set.
    
    Args:
        set_id: str or int
            Rebrickable set id.
        
        part_details: bool
            If set to True part details will be retrieved.
        
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    if '-' not in str(set_id):
        set_id = "%s-1" % set_id
    
    parameters = {
        'inc_part_details': int(part_details) or None,
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    path = config.API_LEGO_URL + "sets/%s/parts/" % set_id
    
    return request(path, parameters)


def get_set_alternates(set_id, page=None, page_size=None, ordering=None, api_key=None):
    """
    Gets details about available alternate builds for specific set.
    
    Args:
        set_id: str or int
            Rebrickable set id.
        
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    if '-' not in str(set_id):
        set_id = "%s-1" % set_id
    
    parameters = {
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    path = config.API_LEGO_URL + "sets/%s/alternates/" % set_id
    
    return request(path, parameters)


def get_themes(page=None, page_size=None, ordering=None, api_key=None):
    """
    Gets details for all available set themes.
    
    Args:
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    path = config.API_LEGO_URL + "themes/"
    
    return request(path, parameters)


def get_theme(theme_id, api_key=None):
    """
    Gets details about specific set theme.
    
    Args:
        theme_id: str or int
            Rebrickable theme id.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {'key': api_key}
    
    path = config.API_LEGO_URL + "themes/%s/" % theme_id
    
    return request(path, parameters)



# Created byMartin.cz
# Copyright (c) Martin Strohalm. All rights reserved.

import re
import time
import urllib.parse
import urllib.request
from . import config

# init last request time
_last_request_time = 0

# define page pattern
_PAGE_PATTERN = re.compile("page=([0-9]+)")


def request(url, parameters={}, post=False):
    """
    Builds the final URL and opens handler.
    
    Args:
        url: str
            Request URL.
        
        parameters: dict
            Request parameters.
        
        post: bool
            If set to True, request will be sent as POST.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    # remove unset parameters
    parameters = {k: v for k, v in parameters.items() if v is not None}
    
    # set default API key
    parameters['key'] = assert_api_key(parameters.get('key', None))
    
    # parse page number
    if 'page' in parameters and parameters['page'].startswith("http"):
        parameters['page'] = _PAGE_PATTERN.findall(parameters['page'])[0]
    
    # prepare options
    options = urllib.parse.urlencode(parameters, doseq=True)
    
    # assert time restrictions
    global _last_request_time
    while True:
        delay = _last_request_time + config.REQUEST_DELAY - time.time()
        if delay <= 0:
            _last_request_time = time.time()
            break
        else:
            time.sleep(delay)
    
    # send request
    if post:
        handle = urllib.request.urlopen(url, options.encode('utf8'))
    else:
        url = "%s?%s" % (url, options)
        handle = urllib.request.urlopen(url)
    
    return handle


def assert_api_key(api_key):
    """Checks given API key and use default."""
    
    # use module key
    if not api_key:
        api_key = config.API_KEY
    
    # check key
    if not api_key:
        raise ValueError("API key not specified. Run rebrick.init() to set your key for all functions.")
    
    return api_key


def assert_user_token(user_token):
    """Checks given user token and use default."""
    
    # use module token
    if not user_token:
        user_token = config.USER_TOKEN
    
    # check token
    if not user_token:
        raise ValueError("User token must be specified. Run rebrick.init() to set your token for all functions.")
    
    return user_token



#  Created byMartin.cz
#  Copyright (c) Martin Strohalm. All rights reserved.

import rebrick

# Before you can start this example be sure to fill in following variables with
# your specific values:

API_KEY = "key"
USER_TOKEN = "token"
USER_NAME = "username"
USER_PASSWORD = "password"

# init Rebrick tool
rb = rebrick.Rebrick(API_KEY, USER_TOKEN, silent=True)

# if user token is not provided on init you can get it later to access user data
rb.login(USER_NAME, USER_PASSWORD)

print("Get part categories:")
data = rb.get_categories()
print(data)
print()

print("Get part category:")
data = rb.get_category(11)
print(data)
print()

print("Get colors:")
data = rb.get_colors()
print(data)
print()

print("Get color:")
data = rb.get_color(4)
print(data)
print()

print("Get element:")
data = rb.get_element(300121)
print(data)
print()

print("Get element ids:")
data = rb.get_element_ids(3001, 4)
print(data)
print()

print("Get element image:")
data = rb.get_element_image(300121)
print(data)
print()

print("Get MOC:")
data = rb.get_moc(24522)
print(data)
print()

print("Get MOC elements:")
data = rb.get_moc_elements(24522)
print(data)
print()

print("Get parts:")
data = rb.get_parts('Bilbo')
print(data)
print()

print("Get part:")
data = rb.get_part(3001)
print(data)
print()

print("Get part colors:")
data = rb.get_part_colors(3001)
print(data)
print()

print("Get part color sets:")
data = rb.get_part_color_sets(3001, 4)
print(data)
print()

print("Get sets:")
data = rb.get_sets(min_year=1982, max_year=1982, min_pieces=20, max_pieces=25)
print(data)
print()

print("Get set:")
data = rb.get_set(6608)
print(data)
print()

print("Get set elements:")
data = rb.get_set_elements(6608)
print(data)
print()

print("Get set themes:")
data = rb.get_set_themes(6608)
print(data)
print()

print("Get set image:")
data = rb.get_set_image(6608)
print(data)
print()

print("Get set alternates:")
data = rb.get_set_alternates(79003)
print(data)
print()

print("Get themes:")
data = rb.get_themes()
print(data)
print()

print("Get theme:")
data = rb.get_theme(73)
print(data)
print()

print("Get user's elements:")
data = rb.get_users_elements(color_id=0)
print(data)
print()

print("Get user's lost elements:")
data = rb.get_users_lost_elements()
print(data)
print()

print("Get user's part lists:")
data = rb.get_users_partlists()
print(data)
print()

print("Get user's part list:")
data = rb.get_users_partlist(21261)  # provide your own list ID
print(data)
print()

print("Get user's part list elements:")
data = rb.get_users_partlist_elements(21261)  # provide your own list ID
print(data)
print()

print("Get user's sets:")
data = rb.get_users_sets()
print(data)
print()

print("Get user's set lists:")
data = rb.get_users_setlists()
print(data)
print()

print("Get user's set list:")
data = rb.get_users_setlist(84012)  # provide your own list ID
print(data)
print()

print("Get user's set list sets:")
data = rb.get_users_setlist_sets(84012)  # provide your own list ID
print(data)
print()



# Created byMartin.cz
# Copyright (c) Martin Strohalm. All rights reserved.

# See the Rebrickable API documentation at https://rebrickable.com/api/v3/docs/

from . import config
from .request import request, assert_user_token


def get_badges(page=None, page_size=None, ordering=None, api_key=None):
    """
    Gets details for all available badges.
    
    Args:
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """

    parameters = {
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    path = config.API_USERS_URL + "badges"
    
    return request(path, parameters)


def get_badge(badge_id, api_key=None):
    """
    Get details about a specific badge.
    
    Args:
        badge_id: str or int
            Rebrickable badge ID.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {'key': api_key}
    
    path = config.API_USERS_URL + "badges/%s/" % badge_id
    
    return request(path, parameters)


def get_build(set_id, user_token=None, api_key=None):
    """
    Finds out how many parts the user needs to build specific set.
    
    Args:
        set_id: str or int
            Rebrickable set ID.
        
        user_token: str or None
            Rebrickable user token. If set to None the one set by
            rebrick.init() is used.rebrick.init() is used.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    if '-' not in str(set_id):
        set_id = "%s-1" % set_id
    
    parameters = {'key': api_key}
    
    user_token = assert_user_token(user_token)
    path = config.API_USERS_URL + "%s/build/%s/" % (user_token, set_id)
    
    return request(path, parameters)


def get_elements(part_id=None, part_cat_id=None, color_id=None, part_details=False, page=None, page_size=None, user_token=None, api_key=None):
    """
    Gets details for all user's elements in partlists and own sets with optional
    filters.
    
    Args:
        part_id: str, int or None
            Rebrickable part ID.
        
        part_cat_id: str, int or None
            Rebrickable category ID.
        
        color_id: str, int or None
            Rebrickable color ID.
        
        part_details: bool
            If set to True part details will be retrieved.
        
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        user_token: str or None
            Rebrickable user token. If set to None the one set by
            rebrick.init() is used.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'part_num': part_id,
        'part_cat_id': part_cat_id,
        'color_id': color_id,
        'inc_part_details': int(part_details) or None,
        'page': page,
        'page_size': page_size,
        'key': api_key}
    
    user_token = assert_user_token(user_token)
    path = config.API_USERS_URL + "%s/allparts/" % user_token
    
    return request(path, parameters)


def get_lost_elements(part_details=False, page=None, page_size=None, ordering=None, user_token=None, api_key=None):
    """
    Gets details for all user's lost elements.
    
    Args:
        part_details: bool
            If set to True part details will be retrieved.
        
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        user_token: str or None
            Rebrickable user token. If set to None the one set by
            rebrick.init() is used.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'inc_part_details': int(part_details) or None,
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    user_token = assert_user_token(user_token)
    path = config.API_USERS_URL + "%s/lost_parts/" % user_token
    
    return request(path, parameters)


def get_partlists(page=None, page_size=None, user_token=None, api_key=None):
    """
    Gets details for all user's parts lists.
    
    Args:
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        user_token: str or None
            Rebrickable user token. If set to None the one set by
            rebrick.init() is used.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'page': page,
        'page_size': page_size,
        'key': api_key}
    
    user_token = assert_user_token(user_token)
    path = config.API_USERS_URL + "%s/partlists/" % user_token
    
    return request(path, parameters)


def get_partlist(list_id, user_token=None, api_key=None):
    """
    Gets details about specific user's part list.
    
    Args:
        list_id: str or int
            Rebrickable part list ID.
        
        user_token: str or None
            Rebrickable user token. If set to None the one set by
            rebrick.init() is used.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {'key': api_key}
    
    user_token = assert_user_token(user_token)
    path = config.API_USERS_URL + "%s/partlists/%s/" % (user_token, list_id)
    
    return request(path, parameters)


def get_partlist_elements(list_id, part_details=False, page=None, page_size=None, ordering=None, user_token=None, api_key=None):
    """
    Gets details about all elements of specific user's part list.
    
    Args:
        list_id: str or int
            Rebrickable part list ID.
        
        part_details: bool
            If set to True part details will be retrieved.
        
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        user_token: str or None
            Rebrickable user token. If set to None the one set by
            rebrick.init() is used.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'inc_part_details': int(part_details) or None,
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    user_token = assert_user_token(user_token)
    path = config.API_USERS_URL + "%s/partlists/%s/parts/" % (user_token, list_id)
    
    return request(path, parameters)


def get_partlist_part_color(list_id, part_id, color_id, part_details=False, user_token=None, api_key=None):
    """
    Gets details about specific part/color combination in user's part list.
    
    Args:
        list_id: str or int
            Rebrickable part list ID.
        
        part_id: str or int
            Rebrickable part ID.
        
        color_id: str or int
            Rebrickable color ID.
        
        part_details: bool
            If set to True part details will be retrieved.
        
        user_token: str or None
            Rebrickable user token. If set to None the one set by
            rebrick.init() is used.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'inc_part_details': int(part_details) or None,
        'key': api_key}
    
    user_token = assert_user_token(user_token)
    path = config.API_USERS_URL + "%s/partlists/%s/parts/%s/%s" % (user_token, list_id, part_id, color_id)
    
    return request(path, parameters)


def get_partlists_elements(search=None, part_id=None, part_cat_id=None, color_id=None, part_details=False, page=None, page_size=None, ordering=None, user_token=None, api_key=None):
    """
    Gets details for all available user's elements in partlists with optional
    filters.
    
    Args:
        search: str, int or None
            Search term e.g. part ID or name.
        
        part_id: str, int or None
            Rebrickable part ID.
        
        part_cat_id: str, int or None
            Rebrickable category ID.
        
        color_id: str, int or None
            Rebrickable color ID.
        
        part_details: bool
            If set to True part details will be retrieved.
        
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        user_token: str or None
            Rebrickable user token. If set to None the one set by
            rebrick.init() is used.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'search': search,
        'part_num': part_id,
        'part_cat_id': part_cat_id,
        'color_id': color_id,
        'inc_part_details': int(part_details) or None,
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    user_token = assert_user_token(user_token)
    path = config.API_USERS_URL + "%s/parts/" % user_token
    
    return request(path, parameters)


def get_profile(user_token=None, api_key=None):
    """
    Gets details about the user.
    
    Args:
        user_token: str or None
            Rebrickable user token. If set to None the one set by
            rebrick.init() is used.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {'key': api_key}
    
    user_token = assert_user_token(user_token)
    path = config.API_USERS_URL + "%s/profile/" % user_token
    
    return request(path, parameters)


def get_sets(search=None, theme_id=None, min_year=None, max_year=None, min_pieces=None, max_pieces=None, page=None, page_size=None, ordering=None, user_token=None, api_key=None):
    """
    Gets details for all user's own sets with optional filters.
    
    Args:
        search: str, int or None
            Search term e.g. set ID or name.
        
        theme_id: str, int or None
            Rebrickable theme ID.
        
        min_year: int or None
            Minimum allowed release year.
        
        max_year: int or None
            Maximum allowed release year.
        
        min_pieces: int or None
            Minimum number of pieces.
        
        max_pieces: int or None
            Maximum number of pieces.
        
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        user_token: str or None
            Rebrickable user token. If set to None the one set by
            rebrick.init() is used.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'search': search,
        'theme_id': theme_id,
        'min_year': min_year,
        'max_year': max_year,
        'min_parts': min_pieces,
        'max_parts': max_pieces,
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    user_token = assert_user_token(user_token)
    path = config.API_USERS_URL + "%s/sets/" % user_token
    
    return request(path, parameters)


def get_setlists(page=None, page_size=None, user_token=None, api_key=None):
    """
    Gets details for all available user's sets lists.
    
    Args:
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        user_token: str or None
            Rebrickable user token. If set to None the one set by
            rebrick.init() is used.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'page': page,
        'page_size': page_size,
        'key': api_key}
    
    user_token = assert_user_token(user_token)
    path = config.API_USERS_URL + "%s/setlists/" % user_token
    
    return request(path, parameters)


def get_setlist(list_id, user_token=None, api_key=None):
    """
    Gets details about specific user's sets list.
    
    Args:
        list_id: str or int
            Rebrickable sets list ID.
        
        user_token: str or None
            Rebrickable user token. If set to None the one set by
            rebrick.init() is used.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'list_id': list_id,
        'key': api_key}
    
    user_token = assert_user_token(user_token)
    path = config.API_USERS_URL + "%s/setlists/%s/" % (user_token, list_id)
    
    return request(path, parameters)


def get_setlist_sets(list_id, page=None, page_size=None, ordering=None, user_token=None, api_key=None):
    """
    Gets list of all sets within specific user's sets list.
    
    Args:
        list_id: str or int
            Rebrickable sets list ID.
        
        page: int or None
            A page number within the paginated result set.
        
        page_size: int or None
            Number of results to return per page.
        
        ordering: str or None
            Specifies the field to use for results ordering.
        
        user_token: str or None
            Rebrickable user token. If set to None the one set by
            rebrick.init() is used.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'list_id': list_id,
        'page': page,
        'page_size': page_size,
        'ordering': ordering,
        'key': api_key}
    
    user_token = assert_user_token(user_token)
    path = config.API_USERS_URL + "%s/setlists/%s/sets/" % (user_token, list_id)
    
    return request(path, parameters)


def get_token(username, password, api_key=None):
    """
    Generates user token to be used for authorizing actions.
    
    Args:
        username: str
            Rebrickable registration username or email.
        
        password: str
            Rebrickable registration password.
        
        api_key: str or None
            Rebrickable API access key. If set to None the one set by
            rebrick.init() is used.
    
    Returns:
        http.client.HTTPResponse
            Server response.
    """
    
    parameters = {
        'username': username,
        'password': password,
        'key': api_key}
    
    path = config.API_USERS_URL + "_token/"
    
    return request(path, parameters, post=True)



# Created byMartin.cz
# Copyright (c) Martin Strohalm. All rights reserved.

import json
import urllib.request
import urllib.error
from . import config
from . import api_lego as lego
from . import api_users as users
from .objects import *


class Rebrick(object):
    """Rebrick tool."""
    
    
    def __init__(self, api_key=None, user_token=None, silent=False):
        """
        Initializes a new instance of rebrick.Rebrick class.
        
        Args:
            api_key: str or None
                Rebrickable API key. If set to None, module global API key is
                used.
            
            user_token:
                Rebrickable user token. If set to None, you need to call login
                method before accessing user account functionality.
            
            silent: bool
                If set to True, all HTTP errors will be silenced and methods
                return None. If set to False, all HTTP errors are raised
                normally.
        """
        
        super(Rebrick, self).__init__()
        
        self._api_key = api_key
        self._user_token = user_token
        self._silent = silent
    
    
    def login(self, username, password):
        """
        Retrieves user login token, which is used to access user account
        functionality.
        
        Args:
            username: str
                Rebrickable login username or email.
            
            password: str
                Rebrickable login password.
        
        Returns:
            str or None
                Returns user token or None if login failed.
        """
        
        # send request
        try:
            response = users.get_token(
                username = username,
                password = password,
                api_key = self._api_key)
        
        except urllib.error.HTTPError as e:
            self._on_error(e)
            return None
        
        # get response data
        data = json.loads(response.read())
        
        # set token
        self._user_token = data.get('user_token', None)
        
        return self._user_token
    
    
    def get_categories(self):
        """
        Gets details for all available part categories.
        
        Returns:
            (rebrick.Category,) or None
                Available part categories.
        """
        
        categories = []
        page = None
        
        while True:
            
            # send request
            try:
                response = lego.get_categories(
                    page = page,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # create categories
            for item in data['results']:
                categories.append(Category.create(item))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return categories
    
    
    def get_category(self, category_id):
        """
        Gets details about specific part category.
        
        Args:
            category_id: str or int
                Rebrickable category ID.
        
        Returns:
            rebrick.Category or None
                Category details.
        """
        
        # send request
        try:
            response = lego.get_category(
                category_id = category_id,
                api_key = self._api_key)
        
        except urllib.error.HTTPError as e:
            self._on_error(e)
            return None
        
        # get response data
        data = json.loads(response.read())
        
        # create category
        return Category.create(data)
    
    
    def get_colors(self):
        """
        Gets details for all available colors.
        
        Returns:
            (rebrick.Color,) or None
                Available colors.
        """
        
        colors = []
        page = None
        
        while True:
            
            # send request
            try:
                response = lego.get_colors(
                    page = page,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # create colors
            for item in data['results']:
                colors.append(Color.create(item))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return colors
    
    
    def get_color(self, color_id):
        """
        Gets details about specific color.
        
        Args:
            color_id: str or int
                Rebrickable color ID.
        
        Returns:
            rebrick.Color or None
                Color details.
        """
        
        # send request
        try:
            response = lego.get_color(
                color_id = color_id,
                api_key = self._api_key)
        
        except urllib.error.HTTPError as e:
            self._on_error(e)
            return None
        
        # get response data
        data = json.loads(response.read())
        
        # create color
        return Color.create(data)
    
    
    def get_element(self, element_id):
        """
        Gets details about specific element.
        
        Args:
            element_id: str or int
                Rebrickable element ID.
        
        Returns:
            rebrick.Element or None
                Element details.
        """
        
        # send request
        try:
            response = lego.get_element(
                element_id = element_id,
                api_key = self._api_key)
        
        except urllib.error.HTTPError as e:
            self._on_error(e)
            return None
        
        # get response data
        data = json.loads(response.read())
        
        # create element
        return Element.create(data)
    
    
    def get_element_ids(self, part_id, color_id):
        """
        Gets element IDs corresponding to given part and color.
        
        Args:
            part_id: str or int
                Rebrickable part ID.
            
            color_id: str or int
                Rebrickable color ID.
        
        Returns:
            (int,) or None
                Element IDs.
        """
        
        # send request
        try:
            response = lego.get_part_color(
                part_id = part_id,
                color_id = color_id,
                api_key = self._api_key)
        
        except urllib.error.HTTPError as e:
            self._on_error(e)
            return None
        
        # get response data
        data = json.loads(response.read())
        
        # get IDs
        return data.get('elements', [])
    
    
    def get_element_image(self, element_id):
        """
        Gets image of specific element.

        Args:
            element_id: str or int
                Rebrickable element ID.

        Returns:
            bytes or None
                Image data.
        """
    
        url = config.ELEMENT_IMG_URL.format(element_id)
        return self.get_image(url)
    
    
    def get_moc(self, moc_id):
        """
        Gets details about specific MOC.
        
        Args:
            moc_id: str or int
                Rebrickable MOC ID.
        
        Returns:
            rebrick.Collection or None
                MOC details.
        """
        
        # send request
        try:
            response = lego.get_moc(
                moc_id = moc_id,
                api_key = self._api_key)
        
        except urllib.error.HTTPError as e:
            self._on_error(e)
            return None
        
        # get response data
        data = json.loads(response.read())
        
        # create set
        return Collection.create(data, 'moc')
    
    
    def get_moc_elements(self, moc_id, part_details=False):
        """
        Gets list of elements for a specific MOC.
        
        Args:
            moc_id: str or int
                Rebrickable MOC ID.
            
            part_details: bool
                If set to True part details will be retrieved.
        
        Returns:
            (rebrick.Element,) or None
                MOC elements.
        """
        
        elements = []
        page = None
        
        while True:
            
            # send request
            try:
                response = lego.get_moc_elements(
                    moc_id = moc_id,
                    part_details = part_details,
                    page = page,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # create elements
            for item in data['results']:
                elements.append(Element.create(item))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return elements
    
    
    def get_parts(self, search=None, part_id=None, part_ids=None, part_cat_id=None, color_id=None, bricklink_id=None, brickowl_id=None, lego_id=None, ldraw_id=None, part_details=False):
        """
        Gets details for all available parts with optional filters.
        
        Args:
            search: str
                Search query.
            
            part_id: str or int
                Rebrickable part ID.
            
            part_ids: (str,), (int,) or None
                Rebrickable part IDs.
            
            part_cat_id: str or int
                Rebrickable part category ID.
            
            color_id: str or int
                Rebrickable color ID.
            
            bricklink_id: str or int
                Bricklink part ID.
            
            brickowl_id: str or int
                BrickOwl part ID.
            
            lego_id: str or int
                LEGO part ID.
            
            ldraw_id: str or int
                LDraw part ID.
            
            part_details: bool
                If set to True part details will be retrieved.
        
        Returns:
            (rebrick.Part,) or None
                Available parts.
        """
        
        parts = []
        page = None
        
        while True:
            
            # send request
            try:
                response = lego.get_parts(
                    search = search,
                    part_id = part_id,
                    part_ids = part_ids,
                    part_cat_id = part_cat_id,
                    color_id = color_id,
                    bricklink_id = bricklink_id,
                    brickowl_id = brickowl_id,
                    lego_id = lego_id,
                    ldraw_id = ldraw_id,
                    part_details = part_details,
                    page = page,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # create parts
            for item in data['results']:
                parts.append(Part.create(item))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return parts
    
    
    def get_part(self, part_id):
        """
        Gets details about specific part.
        
        Args:
            part_id: str or int
                Rebrickable part ID.
        
        Returns:
            rebrick.Part or None
                Part details.
        """
        
        # send request
        try:
            response = lego.get_part(
                part_id = part_id,
                api_key = self._api_key)
        
        except urllib.error.HTTPError as e:
            self._on_error(e)
            return None
        
        # get response data
        data = json.loads(response.read())
        
        # create part
        return Part.create(data)
    
    
    def get_part_colors(self, part_id):
        """
        Gets details about available colors for specific part.
        
        Args:
            part_id: str or int
                Rebrickable part ID.
        
        Returns:
            (rebrick.Color,) or None
                Part colors.
        """
        
        colors = []
        page = None
        
        # get all colors
        lookup = {c.color_id: c for c in self.get_colors()}
        
        while True:
            
            # send request
            try:
                response = lego.get_part_colors(
                    part_id = part_id,
                    page = page,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # get colors
            for item in data['results']:
                if item['color_id'] in lookup:
                    colors.append(lookup[item['color_id']])
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return colors
    
    
    def get_part_color_sets(self, part_id, color_id):
        """
        Gets details about available sets containing specific part/color
        combination.
        
        Args:
            part_id: str or int
                Rebrickable part ID.
            
            color_id: str or int
                Rebrickable color id.
            
        Returns:
            (rebrick.Collection,) or None
                Part color sets.
        """
        
        sets = []
        page = None
        
        while True:
            
            # send request
            try:
                response = lego.get_part_color_sets(
                    part_id = part_id,
                    color_id = color_id,
                    page = page,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # get sets
            for item in data['results']:
                sets.append(Collection.create(item, 'set'))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return sets
    
    
    def get_sets(self, search=None, theme_id=None, min_year=None, max_year=None, min_pieces=None, max_pieces=None):
        """
        Gets a list of all sets with optional filters.
        
        Args:
            search: str
                Search query.
            
            theme_id: str or int
                Rebrickable theme ID.
            
            min_year: int
                Minimum release year.
            
            max_year: int
                Maximum release year.
            
            min_pieces: int
                Minimum number of parts.
            
            max_pieces: int
                Maximum number of parts.
        
        Returns:
            (rebrick.Collection,) or None
                Available sets.
        """
        
        sets = []
        page = None
        
        while True:
            
            # send request
            try:
                response = lego.get_sets(
                    search = search,
                    theme_id = theme_id,
                    min_year = min_year,
                    max_year = max_year,
                    min_pieces = min_pieces,
                    max_pieces = max_pieces,
                    page = page,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # create collections
            for item in data['results']:
                sets.append(Collection.create(item, "set"))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return sets
    
    
    def get_set(self, set_id):
        """
        Gets details about specific set.
        
        Args:
            set_id: str or int
                Rebrickable set ID.
        
        Returns:
            rebrick.Collection or None
                Set details.
        """
        
        # send request
        try:
            response = lego.get_set(
                set_id = set_id,
                api_key = self._api_key)
        
        except urllib.error.HTTPError as e:
            self._on_error(e)
            return None
        
        # get response data
        data = json.loads(response.read())
        
        # create set
        return Collection.create(data, 'set')
    
    
    def get_set_elements(self, set_id, part_details=False):
        """
        Gets list of elements for a specific set.
        
        Args:
            set_id: str or int
                Rebrickable set ID.
            
            part_details: bool
                If set to True part details will be retrieved.
        
        Returns:
            (rebrick.Element,) or None
                Set elements.
        """
        
        elements = []
        page = None
        
        while True:
            
            # send request
            try:
                response = lego.get_set_elements(
                    set_id = set_id,
                    part_details = part_details,
                    page = page,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # create elements
            for item in data['results']:
                elements.append(Element.create(item))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return elements
    
    
    def get_set_themes(self, set_id):
        """
        Gets hierarchy of themes for a specific set.
        
        Args:
            set_id: str or int
                Rebrickable set ID.
        
        Returns:
            (rebrick.Theme,) or None
                Set theme hierarchy.
        """
        
        themes = []
        
        # send request
        try:
            response = lego.get_set(
                set_id = set_id,
                api_key = self._api_key)
        
        except urllib.error.HTTPError as e:
            self._on_error(e)
            return None
        
        # get response data
        data = json.loads(response.read())
        
        # get theme ID
        theme_id = data['theme_id']
        
        # retrieve theme hierarchy
        while theme_id:
            
            # send request
            try:
                response = lego.get_theme(
                    theme_id = theme_id,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # create theme
            theme = Theme.create(data)
            themes.insert(0, theme)
            
            # get parent ID
            theme_id = theme.parent_id
        
        return themes
    
    
    def get_set_image(self, set_id):
        """
        Gets image of specific set.

        Args:
            set_id: str
                Rebrickable set ID.

        Returns:
            bytes or None
                Image data.
        """
        
        if '-' not in str(set_id):
            set_id = "%s-1" % set_id
        
        url = config.SET_IMG_URL.format(set_id)
        return self.get_image(url)
    
    
    def get_set_alternates(self, set_id):
        """
        Gets details about available alternate builds for specific set.
        
        Args:
            set_id: str or int
                Rebrickable set ID.
        
        Returns:
            (rebrick.Collection,) or None
                Alternate sets.
        """
        
        sets = []
        page = None
        
        while True:
            
            # send request
            try:
                response = lego.get_set_alternates(
                    set_id = set_id,
                    page = page,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # create elements
            for item in data['results']:
                sets.append(Collection.create(item, 'moc'))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return sets
    
    
    def get_themes(self):
        """
        Gets details for all available themes.
        
        Returns:
            (rebrick.Theme,) or None
                Available colors.
        """
        
        themes = []
        page = None
        
        while True:
            
            # send request
            try:
                response = lego.get_themes(
                    page = page,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # create themes
            for item in data['results']:
                themes.append(Theme.create(item))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return themes
    
    
    def get_theme(self, theme_id):
        """
        Gets details about specific theme.
        
        Args:
            theme_id: str or int
                Rebrickable theme ID.
        
        Returns:
            rebrick.Theme or None
                Theme details.
        """
        
        # send request
        try:
            response = lego.get_theme(
                theme_id = theme_id,
                api_key = self._api_key)
        
        except urllib.error.HTTPError as e:
            self._on_error(e)
            return None
        
        # get response data
        data = json.loads(response.read())
        
        # create theme
        return Theme.create(data)
    
    
    def get_users_elements(self, part_id=None, part_cat_id=None, color_id=None, part_details=False):
        """
        Gets details for all user's elements in part lists and own sets with
        optional filters.
        
        Args:
            part_id: str, int or None
                Rebrickable part ID.
            
            part_cat_id: str, int or None
                Rebrickable category ID.
            
            color_id: str, int or None
                Rebrickable color ID.
            
            part_details: bool
                If set to True part details will be retrieved.
        
        Returns:
            (rebrick.Element,) or None
                Lost elements.
        """
        
        elements = []
        page = None
        
        while True:
            
            # send request
            try:
                response = users.get_elements(
                    part_id = part_id,
                    part_cat_id = part_cat_id,
                    color_id = color_id,
                    part_details = part_details,
                    page = page,
                    user_token = self._user_token,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # get results
            for item in data['results']:
                elements.append(Element.create(item))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return elements
    
    
    def get_users_lost_elements(self, part_details=False):
        """
        Gets details for all user's lost elements.
        
        Args:
            part_details: bool
                If set to True part details will be retrieved.
        
        Returns:
            (rebrick.Element,) or None
                Lost elements.
        """
        
        elements = []
        page = None
        
        while True:
            
            # send request
            try:
                response = users.get_lost_elements(
                    part_details = part_details,
                    page = page,
                    user_token = self._user_token,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # get results
            for item in data['results']:
                elements.append(Element.create(item))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return elements
    
    
    def get_users_partlists(self):
        """
        Gets a list of all user's part lists.
        
        Returns:
            (rebrick.Partlist,)
                Available part lists.
        """
        
        partlists = []
        page = None
        
        while True:
            
            # send request
            try:
                response = users.get_partlists(
                    page = page,
                    user_token = self._user_token,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # create partlists
            for item in data['results']:
                partlists.append(Partlist.create(item))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return partlists
    
    
    def get_users_partlist(self, list_id):
        """
        Gets details about specific user's parts list.
        
        Args:
            list_id: str or int
                Rebrickable part list ID.
        
        Returns:
            rebrick.Partlist or None
                Part list details.
        """
        
        # send request
        try:
            response = users.get_partlist(
                list_id = list_id,
                user_token = self._user_token,
                api_key = self._api_key)
        
        except urllib.error.HTTPError as e:
            self._on_error(e)
            return None
        
        # get response data
        data = json.loads(response.read())
        
        # create partlist
        return Partlist.create(data)
    
    
    def get_users_partlist_elements(self, list_id, part_details=False):
        """
        Gets list of elements for specific user's parts list.
        
        Args:
            list_id: str or int
                Rebrickable part list ID.
            
            part_details: bool
                If set to True part details will be retrieved.
            
        Returns:
            (rebrick.Element,) or None
                Part list elements.
        """
        
        elements = []
        page = None
        
        while True:
            
            # send request
            try:
                response = users.get_partlist_elements(
                    list_id = list_id,
                    part_details = part_details,
                    page = page,
                    user_token = self._user_token,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # get results
            for item in data['results']:
                elements.append(Element.create(item))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return elements
    
    
    def get_users_sets(self, search=None, theme_id=None, min_year=None, max_year=None, min_pieces=None, max_pieces=None):
        """
        Gets details for all user's own sets with optional filters.
        
        Args:
            search: str
                Search query.
            
            theme_id: str or int
                Rebrickable theme ID.
            
            min_year: int
                Minimum release year.
            
            max_year: int
                Maximum release year.
            
            min_pieces: int
                Minimum number of parts.
            
            max_pieces: int
                Maximum number of parts.
        
        Returns:
            (rebrick.Collection,) or None
                Available sets.
        """
        
        sets = []
        page = None
        
        while True:
            
            # send request
            try:
                response = users.get_sets(
                    search = search,
                    theme_id = theme_id,
                    min_year = min_year,
                    max_year = max_year,
                    min_pieces = min_pieces,
                    max_pieces = max_pieces,
                    page = page,
                    user_token = self._user_token,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # create collections
            for item in data['results']:
                item['set']['quantity'] = item['quantity']
                sets.append(Collection.create(item['set'], "set"))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return sets
    
    
    def get_users_setlists(self):
        """
        Gets a list of all user's set lists.
        
        Returns:
            (rebrick.Setlist,)
                Available set lists.
        """
        
        setlists = []
        page = None
        
        while True:
            
            # send request
            try:
                response = users.get_setlists(
                    page = page,
                    user_token = self._user_token,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # create setlists
            for item in data['results']:
                setlists.append(Setlist.create(item))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return setlists
    
    
    def get_users_setlist(self, list_id):
        """
        Gets details about specific user's set list.
        
        Args:
            list_id: str or int
                Rebrickable set list ID.
        
        Returns:
            rebrick.Setlist or None
                Set list details.
        """
        
        # send request
        try:
            response = users.get_setlist(
                list_id = list_id,
                user_token = self._user_token,
                api_key = self._api_key)
        
        except urllib.error.HTTPError as e:
            self._on_error(e)
            return None
        
        # get response data
        data = json.loads(response.read())
        
        # create setlist
        return Setlist.create(data)
    
    
    def get_users_setlist_sets(self, list_id):
        """
        Gets a list of all user's sets within specific set list.
        
        Args:
            list_id: str or int
                Rebrickable set list ID.
        
        Returns:
            (rebrick.Collection,) or None
                Available sets.
        """
        
        sets = []
        page = None
        
        while True:
            
            # send request
            try:
                response = users.get_setlist_sets(
                    list_id = list_id,
                    page = page,
                    user_token = self._user_token,
                    api_key = self._api_key)
            
            except urllib.error.HTTPError as e:
                self._on_error(e)
                return None
            
            # get response data
            data = json.loads(response.read())
            
            # create collections
            for item in data['results']:
                item['set']['quantity'] = item['quantity']
                sets.append(Collection.create(item['set'], "set"))
            
            # check next page
            if data['next'] is None:
                break
            
            # get next page
            page = data['next']
        
        return sets
    
    
    def get_image(self, url):
        """
        Downloads image from Rebrickable.
        
        Args:
            url: str
                URL of the image to download.
        
        Returns:
            bytes
                Image data.
        """
        
        # make request
        request = urllib.request.Request(url, headers={'User-Agent': 'Rebrick Tool'})
        
        # send request
        try:
            response = urllib.request.urlopen(request)
        
        except urllib.error.HTTPError as e:
            self._on_error(e)
            return None
        
        # get response data
        return response.read()
    
    
    def _on_error(self, error):
        """Process request error."""
        
        if self._silent:
            return
        
        raise error



# Created byMartin.cz
# Copyright (c) Martin Strohalm. All rights reserved.

# define tokens
API_KEY = None
USER_TOKEN = None

# define API urls
API_LEGO_URL = "https://rebrickable.com/api/v3/lego/"
API_USERS_URL = "https://rebrickable.com/api/v3/users/"

# define image urls
ELEMENT_IMG_URL = "https://m.rebrickable.com/media/parts/elements/{0}.jpg"
SET_IMG_URL = "https://m.rebrickable.com/media/sets/{0}.jpg"

# define minimum delay between requests in seconds
REQUEST_DELAY = 0.2



#  Created byMartin.cz
#  Copyright (c) Martin Strohalm. All rights reserved.

import json
import rebrick

# Before you can start this example be sure to fill in following variables with
# your specific values:

API_KEY = "key"
USER_TOKEN = "token"
USER_NAME = "username"
USER_PASSWORD = "password"

# set default tokens for the whole module directly
rebrick.init(API_KEY, USER_TOKEN)

# OR set default tokens for the whole module by login credentials
rebrick.init(API_KEY, USER_NAME, USER_PASSWORD)

print("Get badges:")
response = rebrick.users.get_badges()
print(json.loads(response.read()))
print()


print("Get badge:")
response = rebrick.users.get_badge(1)
print(json.loads(response.read()))
print()

print("Get build:")
response = rebrick.users.get_build(6608)
print(json.loads(response.read()))
print()

print("Get elements:")
response = rebrick.users.get_elements(color_id=0)
print(json.loads(response.read()))
print()

print("Get lost elements:")
response = rebrick.users.get_lost_elements()
print(json.loads(response.read()))
print()

print("Get part lists:")
response = rebrick.users.get_partlists()
print(json.loads(response.read()))
print()

print("Get part list:")
response = rebrick.users.get_partlist(21261)  # provide your own list ID
print(json.loads(response.read()))
print()

print("Get part list elements:")
response = rebrick.users.get_partlist_elements(21261)  # provide your own list ID
print(json.loads(response.read()))
print()

print("Get part list element:")
response = rebrick.users.get_partlist_part_color(21261, 10884, 27)  # provide your own list ID
print(json.loads(response.read()))
print()

print("Get elements in lists:")
response = rebrick.users.get_partlists_elements('dish')
print(json.loads(response.read()))
print()

print("Get profile:")
response = rebrick.users.get_profile()
print(json.loads(response.read()))
print()

print("Get sets:")
response = rebrick.users.get_sets()
print(json.loads(response.read()))
print()

print("Get set lists:")
response = rebrick.users.get_setlists()
print(json.loads(response.read()))
print()

print("Get set list:")
response = rebrick.users.get_setlist(84012)  # provide your own list ID
print(json.loads(response.read()))
print()

print("Get set list sets:")
response = rebrick.users.get_setlist_sets(84012)  # provide your own list ID
print(json.loads(response.read()))
print()

print("Get user token:")
response = rebrick.users.get_token(USER_NAME, USER_PASSWORD)
print(json.loads(response.read()))
print()



# Created byMartin.cz
# Copyright (c) Martin Strohalm. All rights reserved.

# set version
version = (0, 2, 0)

import json
from . import config
from . import api_lego as lego
from . import api_users as users
from .objects import Element, Color, Part, Collection, Theme, Category
from .rebrick import Rebrick


def init(*args):
    """
    Sets API_KEY and USER_TOKEN to be used automatically as defaults for the
    whole rebrick module. The API KEY must be available for all the rebrick
    functions to work. The USER TOKEN is needed for user-specific functions.
    
    In case a single argument is provided it is considered as API_KEY. If two
    arguments are provided they are considered as API_KEY and USER_TOKEN.
    Finally, if three arguments are provided they are considered as API_KEY,
    username and password to retrieve USER_TOKEN from the server.
    """
    
    # set API KEY
    if len(args) == 1:
        config.API_KEY = str(args[0])
    
    # set API KEY and USER TOKEN
    elif len(args) == 2:
        config.API_KEY = str(args[0])
        config.USER_TOKEN = str(args[1])
    
    # set API KEY and retrieve USER TOKEN
    elif len(args) == 3:
        config.API_KEY = str(args[0])
        response = users.get_token(args[1], args[2])
        data = json.loads(response.read())
        config.USER_TOKEN = data.get('user_token', None)
    
    # show help
    else:
        message = "Usage: init(API_KEY) or init(API_KEY, USER_TOKEN) or init(API_KEY, username, password)."
        raise ValueError(message)



# Created byMartin.cz
# Copyright (c) Martin Strohalm. All rights reserved.


class _Entity(object):
    """Provides a base class for all objects."""
    
    
    def __init__(self, **attrs):
        """Initializes a new instance of rebrick.Entity."""
        
        super(_Entity, self).__init__()
        
        # set given attributes
        for name, value in attrs.items():
            if hasattr(self, name):
                setattr(self, name, value)
            else:
                raise AttributeError("Attribute not found! --> %s" % name)
    
    
    def __repr__(self):
        """Gets debug string representation."""
        
        return u"%s(%s)" % (self.__class__.__name__, self.__str__())


class Category(_Entity):
    """
    Represents a Rebrickable part category definition.
    
    Attributes:
        
        category_id: int or None
            Rebrickable category ID.
            
        name: str
            Theme name.
    """
    
    
    def __init__(self, **attrs):
        """Initializes a new instance of rebrick.Category."""
        
        self.category_id = None
        self.name = None
        
        super(Category, self).__init__(**attrs)
    
    
    def __str__(self):
        """Gets standard string representation."""
        
        return u"Category ID: %s %s" % (self.category_id, self.name)
    
    
    @staticmethod
    def create(data):
        """
        Creates a new instance of rebrick.Category from given JSON data.
        
        Args:
            data: dict
                JSON data retrieved from Rebrickable.
        
        Returns:
            rebrick.Category
                Initialized category.
        """
        
        # create category
        return Category(
            category_id = data['id'],
            name = data['name'])


class Collection(_Entity):
    """
    Represents a Rebrickable set or MOC.
    
    Attributes:
        
        type: str
            Collection type as 'set' or 'moc'.
        
        collection_id: str, int or None
            Rebrickable collection ID.
        
        theme_id: int or None
            Rebrickable theme ID.
        
        name: str or None
            Collection name.
        
        year: int or None
            Set release year.
        
        pieces: int or None
            Number of pieces in collection.
        
        url: str or None
            Rebrickable page url.
        
        img_url: str or None
            Rebrickable image url.
        
        designer_name: str or None
            MOC designer name.
        
        designer_url: str or None
            Rebrickable MOC designer page url.
        
        count: int or None
            Collection quantity.
    """
    
    
    def __init__(self, **attrs):
        """Initializes a new instance of rebrick.Collection."""
        
        self.type = None
        
        self.collection_id = None
        self.theme_id = None
        self.name = None
        
        self.designer_name = None
        self.designer_url = None
        
        self.year = None
        self.pieces = None
        
        self.url = None
        self.img_url = None
        
        self.count = None
        
        super(Collection, self).__init__(**attrs)
    
    
    def __str__(self):
        """Gets standard string representation."""
        
        return u"%s ID: %s, %s" % (self.type.upper(), self.collection_id, self.name)
    
    
    @staticmethod
    def create(data, set_type="set"):
        """
        Creates a new instance of rebrick.Collection from given JSON data.
        
        Args:
            data: dict
                JSON data retrieved from Rebrickable.
            
            set_type: str
                Type of a collection to create: 'set' or 'moc'.
        
        Returns:
            rebrick.Collection
                Initialized collection.
        """
        
        # create official set
        if set_type == "set":
            return Collection(
                type = 'set',
                collection_id = data['set_num'],
                theme_id = data['theme_id'],
                name = data['name'],
                year = data['year'],
                pieces = data['num_parts'],
                url = data['set_url'],
                img_url = data['set_img_url'],
                count = data.get('quantity', None))
        
        # create MOC
        if set_type == "moc":
            return Collection(
                type = 'moc',
                collection_id = data['set_num'],
                theme_id = data['theme_id'],
                name = data['name'],
                year = data['year'],
                pieces = data['num_parts'],
                url = data['moc_url'],
                img_url = data['moc_img_url'],
                designer_name = data['designer_name'],
                designer_url = data['designer_url'])


class Color(_Entity):
    """
    Represents a Rebrickable color definition.
    
    Attributes:
        
        color_id: str or None
            Rebrickable color ID.
        
        name: str or None
            Rebrickable color name.
        
        rgb: str or None
            Red, green and blue channels defined as hex code.
        
        is_trans: bool or None
            Marks a transparent color.
         
        external_names: {str:(str,)}
            Available names for external sources.
        
        external_ids: {str:(str,)}
            Available IDs for external sources.
    """
    
    
    def __init__(self, **attrs):
        """Initializes a new instance of rebrick.Color."""
        
        self.color_id = None
        self.name = None
        self.rgb = None
        self.is_trans = None
        self.external_names = {}
        self.external_ids = {}
        
        super(Color, self).__init__(**attrs)
    
    
    def __str__(self):
        """Gets standard string representation."""
        
        return u"Color ID: %s, %s, %s" % (self.color_id, self.name, self.rgb)
    
    
    @staticmethod
    def create(data):
        """
        Creates a new instance of rebrick.Color from given JSON data.
        
        Args:
            data: dict
                JSON data retrieved from Rebrickable.
        
        Returns:
            rebrick.Color
                Initialized color.
        """
        
        # init color
        color = Color(
            color_id = data.get('id'),
            name = data['name'],
            rgb = data['rgb'])
        
        # get external names and IDs
        for name, value in data['external_ids'].items():
            color.external_names[name] = [n for l in value['ext_descrs'] for n in l]
            color.external_ids[name] = value['ext_ids']
        
        return color


class Element(_Entity):
    """
    Represents a Rebrickable element definition as a specific combination of
    part and color.
    
    Attributes:
        
        element_id: str or None
            Rebrickable element ID.
        
        design_id: str or None
            Lego design ID.
        
        part: rebrick.Part
            Part definition.
        
        color: rebrick.Color
            Color definition.
        
        img_url: str or None
            Rebrickable image url.
        
        count: int or None
            Element quantity.
        
        is_spare: bool or None
            Marks a spare element.
    """
    
    
    def __init__(self, **attrs):
        """Initializes a new instance of rebrick.Element."""
        
        self.element_id = None
        self.design_id = None
        
        self.part = None
        self.color = None
        
        self.img_url = None
        
        self.count = None
        self.is_spare = None
        
        super(Element, self).__init__(**attrs)
    
    
    def __str__(self):
        """Gets standard string representation."""
        
        return u"Element ID: %s, %s, %s" % (self.element_id, self.part, self.color)
    
    
    @staticmethod
    def create(data):
        """
        Creates a new instance of rebrick.Element from given JSON data.
        
        Args:
            data: dict
                JSON data retrieved from Rebrickable.
        
        Returns:
            rebrick.Element
                Initialized element.
        """
        
        # create part
        part = Part.create(data['part'])
        
        # create color
        color = Color.create(data['color'])
        
        # create element
        element = Element(
            part = part,
            color = color,
            element_id = data.get('element_id', None),
            design_id = data.get('design_id', None),
            img_url = data.get('element_img_url', None),
            count = data.get('quantity', None),
            is_spare = data.get('is_spare', None))
        
        return element


class Part(_Entity):
    """
    Represents a Rebrickable part definition.
    
    Attributes:
        
        part_id: str or None
            Rebrickable part ID.
        
        category_id: int or None
            Rebrickable category ID.
        
        external_ids: {str:(str,)}
            Available IDs for external sources.
        
        name: str or None
            Rebrickable part name.
        
        year_from: int or None
            Year or the first appearance.
        
        year_to: int or None
            Year or the last appearance.
        
        url: str or None
            Rebrickable page url.
        
        img_url: str or None
            Rebrickable image url.
        
        print_of: str or None
            Rebrickable ID of parent non-printed part.
        
        prints: (str,) or None
            Collection of printed versions IDs.
        
        molds: (str,) or None
            Collection of mold versions IDs.
        
        alternates: (str,) or None
            Collection of alternate versions IDs.
    """
    
    
    def __init__(self, **attrs):
        """Initializes a new instance of rebrick.Part."""
        
        self.part_id = None
        self.category_id = None
        self.external_ids = {}
        self.name = None
        
        self.year_from = None
        self.year_to = None
        
        self.url = None
        self.img_url = None
        
        self.print_of = None
        self.prints = []
        self.molds = []
        self.alternates = []
        
        super(Part, self).__init__(**attrs)
    
    
    def __str__(self):
        """Gets standard string representation."""
        
        return u"Part ID: %s, %s" % (self.part_id, self.name)
    
    
    @staticmethod
    def create(data):
        """
        Creates a new instance of rebrick.Part from given JSON data.
        
        Args:
            data: dict
                JSON data retrieved from Rebrickable.
        
        Returns:
            rebrick.Part
                Initialized part.
        """
        
        return Part(
            part_id = data['part_num'],
            category_id = data['part_cat_id'],
            external_ids = data.get('external_ids', {}),
            name = data['name'],
            year_from = data.get('year_from', None),
            year_to = data.get('year_to', None),
            url = data.get('part_url', None),
            img_url = data.get('part_img_url', None),
            print_of = data.get('print_of', None),
            prints = data.get('prints', []),
            molds = data.get('molds', []),
            alternates = data.get('alternates', []))


class Partlist(_Entity):
    """
    Represents a Rebrickable user's part list.
    
    Attributes:
        
        list_id: str, int or None
            Rebrickable part list ID.
        
        name: str or None
            Part list name.
        
        pieces: int or None
            Number of elements in the list.
    """
    
    
    def __init__(self, **attrs):
        """Initializes a new instance of rebrick.Partlist."""
        
        self.list_id = None
        self.name = None
        self.pieces = None
        
        super(Partlist, self).__init__(**attrs)
    
    
    def __str__(self):
        """Gets standard string representation."""
        
        return u"ID: %s, %s (%s)" % (self.list_id, self.name, self.pieces)
    
    
    @staticmethod
    def create(data):
        """
        Creates a new instance of rebrick.Partlist from given JSON data.
        
        Args:
            data: dict
                JSON data retrieved from Rebrickable.
        
        Returns:
            rebrick.Partlist
                Initialized part list.
        """
        
        return Partlist(
            list_id = data['id'],
            name = data['name'],
            pieces = data['num_parts'])


class Setlist(_Entity):
    """
    Represents a Rebrickable user's set list.
    
    Attributes:
        
        list_id: str, int or None
            Rebrickable set list ID.
        
        name: str or None
            Set list name.
        
        items: int or None
            Number of items in the list.
    """
    
    
    def __init__(self, **attrs):
        """Initializes a new instance of rebrick.Setlist."""
        
        self.list_id = None
        self.name = None
        self.items = None
        
        super(Setlist, self).__init__(**attrs)
    
    
    def __str__(self):
        """Gets standard string representation."""
        
        return u"ID: %s, %s (%s)" % (self.list_id, self.name, self.items)
    
    
    @staticmethod
    def create(data):
        """
        Creates a new instance of rebrick.Setlist from given JSON data.
        
        Args:
            data: dict
                JSON data retrieved from Rebrickable.
        
        Returns:
            rebrick.Setlist
                Initialized set list.
        """
        
        return Setlist(
            list_id = data['id'],
            name = data['name'],
            items = data['num_sets'])


class Theme(_Entity):
    """
    Represents a Rebrickable set theme definition.
    
    Attributes:
        
        theme_id: int or None
            Rebrickable theme ID.
        
        parent_id: int or None
            Rebrickable parent theme ID.
            
        name: str
            Theme name.
    """
    
    
    def __init__(self, **attrs):
        """Initializes a new instance of rebrick.Theme."""
        
        self.theme_id = None
        self.parent_id = None
        self.name = None
        
        super(Theme, self).__init__(**attrs)
    
    
    def __str__(self):
        """Gets standard string representation."""
        
        return u"Theme ID: %s (%s) %s" % (self.theme_id, self.parent_id, self.name)
    
    
    @staticmethod
    def create(data):
        """
        Creates a new instance of rebrick.Theme from given JSON data.
        
        Args:
            data: dict
                JSON data retrieved from Rebrickable.
        
        Returns:
            rebrick.Theme
                Initialized theme.
        """
        
        # create theme
        return Theme(
            theme_id = data['id'],
            parent_id = data.get('parent_id', None),
            name = data['name'])



#  Created by Martin Strohalm

from setuptools import setup, find_packages

# get version
from rebrick import version
version = '.'.join(str(x) for x in version)

# get description
with open("README.md", "r") as fh:
    long_description = fh.read()

# include additional files
package_data = {}

# set classifiers
classifiers = [
    'Development Status :: 3 - Alpha',
    'Programming Language :: Python :: 3 :: Only',
    'Operating System :: OS Independent',
    'Topic :: Utilities',
    'Intended Audience :: Other Audience']

# main setup
setup(
    name = 'rebrick',
    version = version,
    description = 'Python access to Rebrickable API.',
    long_description = long_description,
    long_description_content_type='text/markdown',
    url = 'https://github.com/xxao/rebrick',
    author = 'Martin Strohalm',
    author_email = 'rebrick@bymartin.cz',
    license = 'MIT',
    packages = find_packages(),
    package_data = package_data,
    classifiers = classifiers,
    install_requires = [],
    zip_safe = False)



#  Created byMartin.cz
#  Copyright (c) Martin Strohalm. All rights reserved.

import json
import rebrick

# Before you can start this example be sure to fill in following variable with
# your specific value:

API_KEY = "key"

# set default token for the whole module
rebrick.init(API_KEY)

print("Get part categories:")
response = rebrick.lego.get_categories()
print(json.loads(response.read()))
print()

print("Get part category:")
response = rebrick.lego.get_category(11)
print(json.loads(response.read()))
print()

print("Get colors:")
response = rebrick.lego.get_colors()
print(json.loads(response.read()))
print()

print("Get color:")
response = rebrick.lego.get_color(4)
print(json.loads(response.read()))
print()

print("Get element:")
response = rebrick.lego.get_element(300121)
print(json.loads(response.read()))
print()

print("Get MOC:")
response = rebrick.lego.get_moc(24522)
print(json.loads(response.read()))
print()

print("Get MOC elements:")
response = rebrick.lego.get_moc_elements(24522)
print(json.loads(response.read()))
print()

print("Get parts:")
response = rebrick.lego.get_parts('Bilbo')
print(json.loads(response.read()))
print()

print("Get part:")
response = rebrick.lego.get_part(3001)
print(json.loads(response.read()))
print()

print("Get part color:")
response = rebrick.lego.get_part_color(3001, 4)
print(json.loads(response.read()))
print()

print("Get part colors:")
response = rebrick.lego.get_part_colors(3001)
print(json.loads(response.read()))
print()

print("Get part color sets:")
response = rebrick.lego.get_part_color_sets(3001, 4)
print(json.loads(response.read()))
print()

print("Get sets:")
response = rebrick.lego.get_sets(min_year=1082, max_year=1982, min_pieces=20, max_pieces=25)
print(json.loads(response.read()))
print()

print("Get set:")
response = rebrick.lego.get_set(6608)
print(json.loads(response.read()))
print()

print("Get set elements:")
response = rebrick.lego.get_set_elements(6608)
print(json.loads(response.read()))
print()

print("Get set alternates:")
response = rebrick.lego.get_set_alternates(79003)
print(json.loads(response.read()))
print()

print("Get themes:")
response = rebrick.lego.get_themes()
print(json.loads(response.read()))
print()

print("Get theme:")
response = rebrick.lego.get_theme(73)
print(json.loads(response.read()))
print()
